
' Programm dient zum Stoppen der Zeit, die ein Fahrzeug zwischen zwei Markierungen
' zurcklegt. Die Zeit wird in Geschwindigkeit umgerechnet und zweistellig per Blinkcode
' ausgegeben.

' Die Zeitaufnahme erfolgt durch Taster2 - solange der Taster gedrckt ist, wird die Zeit
' gemessen und bei Loslassen des Tasters ber beide LEDs und einen optionalen Buzzer an
' Portb.4 ausgegeben

' Die Zehnerstelle wird durch Blinken von Led2 angezeigt, die Einerstelle durch Led1.
' Also bei 35 km/h blinkt Led2 (rot) dreimal, dann Led 1 (grn) fnfmal.

' Ausserdem wird das Ergebnis durch Piepstne an PortB.4 ausgegeben, wenn man dort einen
' Piezoschwinger anschliesst. Zuerst erfolgt die Ausgabe der Zehnerstelle in hoher Tonlage,
' dann die der Einerstelle in tieferer Tonlage.

' Nullen werden durch einen langen Blinkimpuls bzw.einen langen Ton angezeigt.


' Umrechnung der Zeit in Geschwindigkeit:
'   Speed [m/s] = Distanz [m] /  T [sec]
'   Speed [km/h] = 3,6 * Distanz [m] / T [sec]
'                = 3,6 * Distanz [dm] * 0,1 / T [10 msec] * 0,01
'                = 36 * Distanz [dm] / T [10ms]
'                =  Faktor / T [10ms]
' Um Fliesskommazahlen zu vermeiden, rechnen wir in Dezimetern und 10 ms-Intervallen

' Zu Beginn ist die Eingabe der Entfernung zwischen den Wegmarkierungen erforderlich. Sie
' erfolgt in 0,5m- Schritten. Erst wird Taster1 und dann zustzlich Taster2 gedrckt und
' gehalten. Beide LEDs beginnen zu blinken, Jedes Blinken steht fr 0,5m. Nach Erreichen der
' erforderlichen Anzahl lt man die beiden Tasten los.

' Verwendeter Compiler: BASCOM 2.0.5 DEMO

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Taktfrequenz: 1,2MHz
$hwstack = 8                                                ' nur max 4 geschachtelte Unterprogramme; spart SRAM
$swstack = 0                                                ' nicht bentigt; spart SRAM
$framesize = 0                                              ' dito


'---- vom Nutzer anzupassen

Const Kurz = 50                                             ' daraus ergibt sich die Blinkgeschwindigkeit
Const Pause = 450
Const Tonhoehe = 150                                        ' Tonhhen
Const Tonhoehe2 = 180

'----

Const Lang = Kurz * 10

Buzzer Alias Portb.4
Led1 Alias Portb.1                                          ' grn, rechts
Led2 Alias Portb.3                                          ' rot, links
Taster1 Alias Pinb.0                                        ' links
Taster2 Alias Pinb.2                                        ' rechts


Dim I As Byte
Dim N As Byte
Dim Zeit As Word
Dim Speed As Word
Dim Distanz As Word
Dim Faktor As Word
Dim Zehner As Word                                          ' Zehnerstelle des umgerechneten Analogwertes
Dim Einer As Word                                           ' Einerstelle des umgerechneten Analogwertes

Clkpr = 128
Clkpr = 3                                                   ' Takt herabsetzen, falls der ATtiny mit 9,6 MHz Takt luft
Clkpr = 3

Acsr.acd = 1                                                ' Analog-Komparator ausschalten, spart etwas Strom
Gimsk.5 = 1                                                 ' Pin Change Interrupt Enable
Sreg.7 = 1                                                  ' Global Interrupt Enable
Pcmsk.2 = 1                                                 ' Pin Change Interrupt PB2 aktivieren
Ddrb = &B00011010                                           ' PortB.1 und Portb.3 Ausgang
Portb = &B00000101                                          ' PullUp-Widerstnde an Pinb.0 und Pinb.2
Didr0 = &B00011010                                          ' nicht benoetigte Digital- Eingnge abschalten


'
'-------------------------------------------------------------------------------
' Hauptprogramm:


 Do

    If Taster2 = 1 Then                                     ' unterdrckt die Ausgabe "00" unmittelbar nach Anlegen der Batteriespannung
        Goto Schlafen
    End If

    If Taster1 = 0 Then                                     ' Wirkungsweise: PCINT2 (Taster2) weckt den Controller auf. Nur wenn
        Gosub Einstellen                                    ' er Taster1 bereits gedrckt vorfindet, wird nach "Einstellen" gesprungen                                       ' wird in die Einstell-Routine verzweigt
        Goto Schlafen
    End If

    Gimsk.5 = 0                                             ' Pin Change Interrupt ausschalten
    Zeit = 0
    Do
        Incr Zeit
        Waitms 10
    Loop Until Taster2 = 1

    Faktor = Distanz * 36
    Speed = Faktor / Zeit

    If Speed > 99 Then                                      ' berlauf abfangen
        Speed = 99
    End If

    Zehner = Speed / 10                                     ' Zerlegung in Einzelzahlen
    Einer = Speed Mod 10

    Wait 1
    N = Zehner                                              ' Zehnerstelle des Messwerts
    Gosub Beepandblink_z                                    ' wir senden die Zehnerstelle
    Wait 1
    N = Einer                                               ' Einerstelle  des Messwerts
    Gosub Beepandblink_e                                    ' wir senden die Einerstelle

 Schlafen:

    Gimsk.5 = 1                                             ' Pin Change Interrupt wieder einschalten
    Powerdown

 Loop


 Beepandblink_z:

     If N = 0 Then
         Led2 = 1
         Sound Buzzer , Lang , Tonhoehe
         Led2 = 0
     End If

     For I = 1 To N
         Led2 = 1
         Sound Buzzer , Kurz , Tonhoehe
         Led2 = 0
         Waitms Pause
     Next
 Return

 Beepandblink_e:

     If N = 0 Then
         Led1 = 1
         Sound Buzzer , Lang , Tonhoehe2
         Led1 = 0
     End If

     For I = 1 To N
         Led1 = 1
         Sound Buzzer , Kurz , Tonhoehe2
         Led1 = 0
         Waitms Pause
     Next
 Return



 Einstellen:

   Distanz = 0
   Do
       Distanz = Distanz + 5                                ' 0,5m-Schritte
       Led1 = 1
       Led2 = 1
       Waitms 100
       Led1 = 0
       Led2 = 0
       Waitms 400
   Loop Until Taster1 = 1 And Taster2 = 1

 Return

 End