
' Programm dient zum Stoppen der Zeit, die ein Fahrzeug zwischen zwei Markierungen
' zurcklegt. Die Zeit wird in Geschwindigkeit umgerechnet und zweistellig per Morsezeichen
' ausgegeben.

' Die Zeitaufnahme erfolgt durch Taster2 - solange der Taster gedrckt ist, wird die Zeit
' gemessen und bei Loslassen des Tasters das Ergebnis in Morse ausgegeben. Die Ausgabe
' erfolgt ber beide LEDs und einen optionalen Buzzer an Portb.4

' Umrechnung der Zeit in Geschwindigkeit:
'   Speed [m/s] =/ Distanz [m] /  T [sec]
'   Speed [km/h] = 3,6 * Distanz [m] / T [sec]
'                = 3,6 * Distanz [dm] * 0,1 / T [10 msec] * 0,01
'                = 36 * Distanz [dm] / T [10ms]
'                =  Faktor / T [10ms]
' Um Fliesskommazahlen zu vermeiden, rechnen wir in Dezimetern und 10 ms-Intervallen

' Zu Beginn ist die Eingabe der Entfernung zwischen den Wegmarkierungen erforderlich. Sie
' erfolgt in 0,5m- Schritten. Erst wird Taster1 und dann zustzlich Taster2 gedrckt und
' gehalten. Beide LEDs beginnen zu blinken, Jedes Blinken steht fr 0,5m. Nach Erreichen der
' erforderlichen Anzahl lt man die beiden Tasten los.


' Erluterungen zum Morseteil:
'
' Codierung der Morsezeichen: 1 Byte pro Zeichen, niederwertiges Bit zuerst.
' Ist der Bit-Wert 0: Morsepunkt (dit)
' Ist der Bit-Wert 1: Moersestrich (dah)
' Da Morsezeichen unterschiedlich lang sind, ist das Ende-Zeichen ebenfalls ein 1-Bit.
'
' Beispiel Morsezeichen "a" (dit dah): &B00000110
' Beispiel Morsezeichen "9" (dah dah dah dah dit): &B00101111

' Auslesen der codierten Zeichen:
' niederwertigstes Bit auslesen, Byte um 1 Stelle nach rechts schieben, niederwertigstes Bit
' auslesen. Ist der Gesamtwert des Bytes nur noch 1, ist das Zeichen komplett.

' Verwendeter Compiler: BASCOM 2.0.5 DEMO

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Taktfrequenz: 1,2MHz
$hwstack = 8                                                ' nur max 4 geschachtelte Unterprogramme; spart SRAM
$swstack = 0                                                ' nicht bentigt; spart SRAM
$framesize = 0                                              ' dito


'----  vom Nutzer anzupassen

Const Dit = 120                                             ' Lnge eines Punkts, daraus ergibt sich die Morsegeschwindigkeit
Const Tonhoehe = 150                                        ' Tonhhe

'----

Const Msg_laenge = 11                                       ' Anzahl der DataBytes; maximal 47
Const Dah = Dit * 3                                         ' Lnge eines Strichs
Const Dit_pause = Dit
Const Dah_pause = Dit_pause * 5


Buzzer Alias Portb.4
Led1 Alias Portb.1                                          ' grn, rechts
Led2 Alias Portb.3                                          ' rot, links
Taster1 Alias Pinb.0                                        ' links
Taster2 Alias Pinb.2                                        ' rechts


Dim I As Byte                                               ' wird in Zhlschleife verwendet
Dim N As Byte                                               ' Aktueller Morsebuchstabe
Dim M As Word                                               ' wird frs Auslesen der Morsebits verwendet
Dim L As Byte                                               ' niederwertigstes Bit von M verwendet
Dim Zeit As Word                                            '
Dim Distanz As Word
Dim Faktor As Word
Dim Speed As Word
Dim Zehner As Word                                          ' Zehnerstelle des umgerechneten Analogwertes
Dim Einer As Word                                           ' Einerstelle des umgerechneten Analogwertes
Dim Msg(msg_laenge) As Byte                                 ' Feld fr die 10 Morse-Ziffern


Clkpr = 128
Clkpr = 3                                                   ' Takt herabsetzen, falls der ATtiny mit 9,6 MHz Takt luft
Clkpr = 3

Acsr.acd = 1                                                ' Analog-Komparator ausschalten, spart etwas Strom
Gimsk.5 = 1                                                 ' Pin Change Interrupt Enable
Sreg.7 = 1                                                  ' Global Interrupt Enable
Pcmsk.2 = 1                                                 ' Pin Change Interrupt PB2 Enable
Ddrb = &B00011010                                           ' Portb.1, Portb.3, Portb.4 Ausgang
Portb = &B00000101                                          ' PullUp-Widerstnde an Pinb.0 und Pinb.2
Didr0 = &B00011010                                          ' nicht benoetigte Digital- Eingnge abschalten



Restore Message                                             ' Feld einlesen
For I = 1 To Msg_laenge
Read Msg(i)
Next

'
'-------------------------------------------------------------------------------
' Hauptprogramm:



 Do

    If Taster2 = 1 Then                                     ' unterdrckt die Ausgabe "00" unmittelbar nach Anlegen der Batteriespannung
        Goto Schlafen
    End If

    If Taster1 = 0 Then                                     ' Wirkungsweise: PCINT2 (Taster2)weckt den Controller auf. Nur wenn
        Gosub Einstellen                                    ' er Taster1 bereits gedrckt vorfindet, wird nach "Einstellen" gesprungen                                       ' wird in die Einstell-Routine verzweigt
        Goto Schlafen
    End If

    Gimsk.5 = 0                                             ' Pin Change Interrupt ausschalten
    Zeit = 0
    Do
        Incr Zeit
        Waitms 10
    Loop Until Taster2 = 1

    Faktor = Distanz * 36
    Speed = Faktor / Zeit

    If Speed > 99 Then                                      ' berlauf abfangen
        Speed = 99
    End If

    Zehner = Speed / 10                                     ' Zerlegung in Einzelzahlen
    Einer = Speed Mod 10

    Wait 1
    N = Zehner + 1                                          ' Zehnerstelle des Messwerts
      Gosub Morse                                           ' wir senden die Zehnerstelle
    Waitms Dah_pause
    N = Einer + 1                                           ' Einerstelle  des Messwerts
      Gosub Morse                                           ' wir senden die Einerstelle

 Schlafen:

    Gimsk.5 = 1                                             ' Pin Change Interrupt wieder einschalten
    Powerdown

 Loop


' Morsezeichenerzeugung:

 Morse:

     M = Msg(n)                                             ' aktuelles Zeichen aus Kette rauspicken

     For I = 1 To 8
       If M = 1 Then                                        ' Das Byte hat nur noch den Wert 1; Zeichenende!
          Goto Sign_end
       End If
       Led1 = 1
       Led2 = 1
       If M.0 = 1 Then                                      ' ist Bit.0 = 1   -> dah
           Sound Buzzer , Dah , Tonhoehe
         Else
           Sound Buzzer , Dit , Tonhoehe                    ' ist das Bit 0 -> dit
       End If
       Led1 = 0
       Led2 = 0

       Waitms Dit_pause                                     ' Pause innerhalb des Morsezeichens
       Shift M , Right , 1                                  ' Bits um eine Stelle nach rechts shiften
     Next I
     Sign_end:

 Return


 Einstellen:

   Distanz = 0
   Do
       Distanz = Distanz + 5                                ' 0,5m-Schritte
       Led1 = 1
       Led2 = 1
       Waitms 100
       Led1 = 0
       Led2 = 0
       Waitms 400
   Loop Until Taster1 = 1 And Taster2 = 1

 Return


End


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Codierung der Morsezeichen. Bei einem Prozessor mit viel SRAM wrde man die  Zeichentabelle
' in ein Array einlesen, den in Morse zu codierenden Text in einen String packen und diesen
' String zeichenweise abarbeiten.
'
' Da der Attiny13 nur wenig SRAM hat, ist der Weg hier anders:  Der zu codierende Text ist ein
' Array mit 10 Elementen, in dem die auszusendenden Zeichen stehen, allerdings in einer leicht
' lesbaren Schreibweise.

' Die bersetzungstabelle ist mit Konstanten realisiert. Nur die wirklich bentigten Zeichen werden
' (beim kompilieren des Programmes) umgewandelt.


Const #0 = &B00111111
Const #1 = &B00111110
Const #2 = &B00111100
Const #3 = &B00111000
Const #4 = &B00110000
Const #5 = &B00100000
Const #6 = &B00100001
Const #7 = &B00100011
Const #8 = &B00100111
Const #9 = &B00101111


Message:

Data #0 , #1 , #2 , #3 , #4 , #5 , #6 , #7 , #8 , #9